// **************************************************************************
// * This file is part of the FreeFileSync project. It is distributed under *
// * GNU General Public License: http://www.gnu.org/licenses/gpl.html       *
// * Copyright (C) Zenju (zenju AT gmx DOT de) - All Rights Reserved        *
// **************************************************************************

#include <iostream>
#include <fstream>
#include <string>
#include <zen/string_tools.h>


std::string formatRcVersion(const std::string& version)
{
    std::vector<std::string> digits;

    if (std::find(version.begin(), version.end(), '.') != version.end())
        digits = zen::split(version, '.');
    else if (std::find(version.begin(), version.end(), ',') != version.end())
        digits = zen::split(version, ',');
    else
        return std::string();

    digits.resize(4, "0"); //final format: 1,0,0,0
    return digits[0] + "," + digits[1] + "," + digits[2] + "," + digits[3];
}


std::string formatNsisRcVersion(const std::string& version)
{
    return zen::replaceCpy(formatRcVersion(version), ',', '.');  //final format: 1.0.0.0
}


int main(int args, char** argv)
{
    std::cout << "Enter new version number, e.g. 1.17: ";

    const std::string versionNumber = [&]() -> std::string
    {
        std::string tmp;
        std::cin >> tmp;
        return tmp;
    }();

    //################################################################################
    //write version number for update-notifier
    std::ofstream versionFile("latest_version.txt");
    versionFile << versionNumber;

    //################################################################################
    //write C++ include for version number
    std::ofstream versionIncl("version.h");
    versionIncl << "#ifndef VERSION_HEADER_434343489702544325" << std::endl;
    versionIncl << "#define VERSION_HEADER_434343489702544325" << std::endl;
    versionIncl << std::endl;
    versionIncl << "namespace zen" << std::endl;
    versionIncl << "{" << std::endl;
    versionIncl << "    const wchar_t currentVersion[] = L\"" << versionNumber << "\"; //internal linkage!" << std::endl;
    versionIncl << "}" << std::endl;
    versionIncl << std::endl;
    versionIncl << "#endif" << std::endl;

    //################################################################################
    //write Windows resource file include
    std::ofstream resourceIncl("version.rc");
    resourceIncl << "#define FREEFILESYNC_VER     " << formatRcVersion(versionNumber) << "\n";
    resourceIncl << "#define FREEFILESYNC_VER_STR \"" << versionNumber << "\\0\"" << "\n";;

    //################################################################################
    //create header with version information for NSIS installer
    std::ofstream versionNsis("version.nsh");
    versionNsis << "Name \"FreeFileSync " << versionNumber << "\"" << std::endl;
    versionNsis << "OutFile \"FreeFileSync_" << versionNumber << "_setup.exe\"" << std::endl;
    versionNsis << "!define FFS_Version_Str \"FreeFileSync " << versionNumber << "\"" << std::endl;
    versionNsis << "!define FFS_Version_Id \"" << versionNumber << "\"" << std::endl;
    versionNsis << "!define FFS_RC_Version_Id \"" << formatNsisRcVersion(versionNumber) << "\"" << std::endl;

    //################################################################################

    std::cout << "\nOperation successful!\n" << std::endl;
    system("pause");
    return 0;
}
